@@ -1,7 +1,7 @@
 /* Inadyn is a small and simple dynamic DNS (DDNS) client
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
- * Copyright (C) 2010-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2020  Joachim Nilsson <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,10 +35,12 @@
 #include "ssl.h"
 
 int    once = 0;
+int    force = 0;		/* Only allowed with 'once' */
 int    ignore_errors = 0;
 int    startup_delay = DDNS_DEFAULT_STARTUP_SLEEP;
 int    allow_ipv6 = 0;
 int    secure_ssl = 1;		/* Strict cert validation by default */
+int    broken_rtc = 0;		/* Validate certificate time by default */
 char  *ca_trust_file = NULL;	/* Custom CA trust file/bundle PEM format */
 int    verify_addr = 1;
 char  *prognm = NULL;
@@ -50,6 +52,7 @@ char  *config = NULL;
 char  *cache_dir = NULL;
 char  *script_cmd = NULL;
 char  *script_exec = NULL;
+int exec_mode = EXEC_MODE_COMPAT;
 char  *pidfile_name = NULL;
 uid_t  uid = 0;
 gid_t  gid = 0;
@@ -156,9 +159,23 @@ static int drop_privs(void)
 
 static void parse_privs(char *user)
 {
-	char *group = strstr(user, ":");
 	struct passwd *pw;
+	char *group;
 
+	if (!user) {
+		pw = getpwuid(geteuid());
+		if (!pw)
+			user = getenv("LOGNAME");
+		else
+			user = pw->pw_name;
+	}
+
+	if (!user) {
+		logit(LOG_INFO, "Cannot figure out username");
+		return;
+	}
+
+	group = strstr(user, ":");
 	if (group)
 		*group++ = 0;
 
@@ -207,7 +224,7 @@ static int compose_paths(void)
 		snprintf(cache_dir, len, "%s/cache/%s", LOCALSTATEDIR, ident);
 
 		if (access(cache_dir, W_OK)) {
-			char *home;
+			char *home, *tmp;
 
 			home = getenv("HOME");
 			if (!home) {
@@ -217,9 +234,13 @@ static int compose_paths(void)
 
 			/* Fallback cache dir: $HOME + "/.cache/" + "inadyn" */
 			len = strlen(home) + strlen(ident) + 10;
-			cache_dir = realloc(cache_dir, len);
-			if (!cache_dir)
+			tmp = realloc(cache_dir, len);
+			if (!tmp){
+				free(cache_dir);
 				goto nomem;
+			} else {
+				cache_dir = tmp;
+			}
 
 			snprintf(cache_dir, len, "%s/.cache/%s", home, ident);
 			if (mkdir(cache_dir, 0755) && EEXIST != errno) {
@@ -238,17 +259,21 @@ static int usage(int code)
 
 	DO(compose_paths());
 	if (pidfile_name[0] != '/')
-		snprintf(pidfn, sizeof(pidfn), "%s/run/%s.pid", LOCALSTATEDIR, pidfile_name);
+		snprintf(pidfn, sizeof(pidfn), "%s/%s.pid", RUNSTATEDIR, pidfile_name);
 	else
 		snprintf(pidfn, sizeof(pidfn), "%s", pidfile_name);
 
 	fprintf(stderr, "Usage:\n %s [1hnsv] [-c CMD] [-e CMD] [-f FILE] [-l LVL] [-p USR:GRP] [-t SEC]\n\n"
-		" -1, --once                     Run once, then exit regardless of status\n"
+		" -1, --once                     Run only once, updates if too old or unknown\n"
+		"     --force                    Force update, even if address has not changed\n"
 		"     --cache-dir=PATH           Persistent cache dir of IP sent to providers.\n"
 		"                                Default use ident NAME: %s/\n"
 		" -c, --cmd=/path/to/cmd         Script or command to run to check IP\n"
 		" -C, --continue-on-error        Ignore errors from DDNS provider\n"
-		" -e, --exec=/path/to/cmd        Script to run on successful DDNS update\n"
+		" -e, --exec=/path/to/cmd        Script to run on DDNS update\n"
+		"     --exec-mode=MODE           Set script run mode: compat, event:\n"
+		"                                - compat: successful DDNS update only, default\n"
+		"                                - event: any update status\n"
 		"     --check-config             Verify syntax of configuration file and exit\n"
 		" -f, --config=FILE              Use FILE name for configuration, default uses\n"
 		"                                ident NAME: %s\n"
@@ -259,6 +284,7 @@ static int usage(int code)
 		" -l, --loglevel=LEVEL           Set log level: none, err, info, notice*, debug\n"
 		" -n, --foreground               Run in foreground with logging to stdout/stderr\n"
 		" -p, --drop-privs=USER[:GROUP]  Drop privileges after start to USER:GROUP\n"
+		"     --no-pidfile               Do not create PID file, for use with systemd\n"
 		" -P, --pidfile=FILE             File to store process ID for signaling %s\n"
 		"                                Default uses ident NAME: %s\n"
 		" -s, --syslog                   Log to syslog, default unless --foreground\n"
@@ -296,10 +322,12 @@ int main(int argc, char *argv[])
 	int background = 1;
 	struct option opt[] = {
 		{ "once",              0, 0, '1' },
+		{ "force",             0, 0, '4' },
 		{ "cache-dir",         1, 0, 128 },
 		{ "cmd",               1, 0, 'c' },
 		{ "continue-on-error", 0, 0, 'C' },
 		{ "exec",              1, 0, 'e' },
+		{ "exec-mode",         1, 0, 130 },
 		{ "config",            1, 0, 'f' },
 		{ "check-config",      0, 0, 129 },
 		{ "iface",             1, 0, 'i' },
@@ -307,6 +335,7 @@ int main(int argc, char *argv[])
 		{ "loglevel",          1, 0, 'l' },
 		{ "help",              0, 0, 'h' },
 		{ "foreground",        0, 0, 'n' },
+		{ "no-pidfile",        0, 0, 'N' },
 		{ "pidfile",           1, 0, 'P' },
 		{ "drop-privs",        1, 0, 'p' },
 		{ "syslog",            0, 0, 's' },
@@ -316,13 +345,20 @@ int main(int argc, char *argv[])
 	};
 	ddns_t *ctx = NULL;
 
+	/* Set up initial values for uid + gid */
+	parse_privs(NULL);
+
 	prognm = ident = progname(argv[0]);
-	while ((c = getopt_long(argc, argv, "1c:Ce:f:h?i:I:l:np:P:st:v", opt, NULL)) != EOF) {
+	while ((c = getopt_long(argc, argv, "1c:Ce:f:h?i:I:l:nNp:P:st:v", opt, NULL)) != EOF) {
 		switch (c) {
 		case '1':	/* --once */
 			once = 1;
 			break;
 
+		case '4':	/* -- force */
+			force = 1;
+			break;
+
 		case 128:	/* --cache-dir=PATH */
 			cache_dir = strdup(optarg);
 			break;
@@ -339,6 +375,15 @@ int main(int argc, char *argv[])
 			script_exec = optarg;
 			break;
 
+		case 130:	/* --exec-mode=MODE */
+			if (!strcmp(optarg, "event"))
+				exec_mode = EXEC_MODE_EVENT;
+			else if (!strcmp(optarg, "compat"))
+				exec_mode = EXEC_MODE_COMPAT;
+			else
+				return usage(1);
+			break;
+
 		case 'f':	/* --config=FILE */
 			config = strdup(optarg);
 			break;
@@ -368,7 +413,12 @@ int main(int argc, char *argv[])
 			use_syslog--;
 			break;
 
+		case 'N':	/* --no-pidfile */
+			optarg = "";
+			/* fallthrough */
 		case 'P':	/* --pidfile=NAME */
+			if (pidfile_name)
+				free(pidfile_name);
 			pidfile_name = strdup(optarg);
 			break;
 
@@ -402,8 +452,10 @@ int main(int argc, char *argv[])
 	if (check_config) {
 		char pidfn[80];
 
-		if (pidfile_name[0] != '/')
-			snprintf(pidfn, sizeof(pidfn), "%s/run/%s.pid", LOCALSTATEDIR, pidfile_name);
+		if (pidfile_name[0] == 0)
+			strlcpy(pidfn, "<none>", sizeof(pidfn));
+		else if (pidfile_name[0] != '/')
+			snprintf(pidfn, sizeof(pidfn), "%s/%s.pid", RUNSTATEDIR, pidfile_name);
 		else
 			snprintf(pidfn, sizeof(pidfn), "%s", pidfile_name);
 
@@ -442,9 +494,13 @@ int main(int argc, char *argv[])
 	log_init(ident, use_syslog < 1 ? 0 : 1, background);
 
 	/* Check permission to write PID and cache files */
-	if (!once)
+	if (!once) {
 		DO(os_check_perms());
 
+		/* Only allowed with --once */
+		force = 0;
+	}
+
 	if (drop_privs()) {
 		logit(LOG_WARNING, "Failed dropping privileges: %s", strerror(errno));
 		rc = RC_OS_CHANGE_PERSONA_FAILURE;
